<?php

declare(strict_types=1);

namespace Pdahal\Xmpp;

use Pdahal\Xmpp\AuthTypes\Authenticable;
use Pdahal\Xmpp\AuthTypes\Plain;
use Pdahal\Xmpp\Loggers\Loggable;
use Pdahal\Xmpp\Loggers\Logger;
use Psr\Log\InvalidArgumentException;

class Options
{
    /**
     * Hostname of XMPP server.
     */
    protected string $host = '';

    /**
     * Domain Name of XMPP server.
     */
    protected string $domain = '';

    /**
     * XMPP server port. Usually 5222.
     */
    protected int $port = 5222;

    /**
     * Protocol used for socket connection, defaults to TCP.
     */
    protected string $protocol = 'tcp';

    /**
     * Username to authenticate on XMPP server.
     */
    protected string $username = '';

    /**
     * Password to authenticate on XMPP server.
     */
    protected string $password = '';

    /**
     * XMPP resource.
     */
    protected string $resource = '';

    /**
     * Custom logger interface.
     */
    protected ?Loggable $logger = null;

    /**
     * Use TLS if available.
     */
    protected bool $useTls = true;

    /**
     * Auth type (Authentication/AuthTypes/).
     */
    protected ?Authenticable $authType = null;

    public function getHost(): string
    {
        if (!$this->host) {
            $this->getLogger()->error(__METHOD__.'::'.__LINE__.
                ' No host found, please set the host variable');

            throw new InvalidArgumentException();
        }

        return $this->host;
    }

    public function setHost(string $host): Options
    {
        $this->host = trim($host);

        return $this;
    }

    public function setDomain(string $domain): Options
    {
        $this->domain = $domain;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getDomain(): string
    {
        if ($this->domain == '') {
            return $this->getHost();
        }

        return $this->domain;
    }

    public function getPort(): int
    {
        return $this->port;
    }

    public function setPort(int $port): Options
    {
        $this->port = $port;

        return $this;
    }

    public function getUsername(): string
    {
        if ($this->username == '') {
            $this->getLogger()->error(__METHOD__.'::'.__LINE__.
                ' No username found, please set the username variable');

            throw new InvalidArgumentException();
        }

        return $this->username;
    }

    /**
     * Try to assign a resource if it exists. If bare JID is forwarded, this will default to your username.
     */
    public function setUsername(string $username): Options
    {
        $usernameResource = explode('/', $username);

        if (count($usernameResource) > 1) {
            $this->setResource($usernameResource[1]);
            $username = $usernameResource[0];
        }

        $this->username = trim($username);

        return $this;
    }

    /**
     * @throws InvalidArgumentException
     */
    public function getPassword(): string
    {
        if ($this->password == '') {
            $this->getLogger()->error(__METHOD__.'::'.__LINE__.
                ' No password found, please set the password variable');

            throw new InvalidArgumentException();
        }

        return $this->password;
    }

    public function setPassword(string $password): Options
    {
        $this->password = $password;

        return $this;
    }

    public function getResource(): string
    {
        if ($this->resource == '') {
            $this->resource = 'Pdahal_machine_'.time();
        }

        return $this->resource;
    }

    public function setResource(string $resource): Options
    {
        $this->resource = trim($resource);

        return $this;
    }

    public function getProtocol(): string
    {
        return $this->protocol;
    }

    public function setProtocol(string $protocol): Options
    {
        $this->protocol = $protocol;

        return $this;
    }

    public function fullSocketAddress(): string
    {
        $protocol = $this->getProtocol();
        $host = $this->getHost();
        $port = $this->getPort();

        return "{$protocol}://{$host}:{$port}";
    }

    public function fullJid(): string
    {
        $username = $this->getUsername();
        $resource = $this->getResource();
        $host = $this->getHost();

        return "{$username}@{$host}/{$resource}";
    }

    public function bareJid(): string
    {
        $username = $this->getUsername();
        $host = $this->getHost();

        return "{$username}@{$host}";
    }

    public function setLogger(Loggable $logger): void
    {
        $this->logger = $logger;
    }

    public function getLogger(): Loggable
    {
        if ($this->logger === null) {
            $this->logger = new Logger();
        }

        return $this->logger;
    }

    public function setUseTls(bool $enable): void
    {
        $this->useTls = $enable;
    }

    public function usingTls(): bool
    {
        return $this->useTls;
    }

    public function getAuthType(): Authenticable
    {
        if ($this->authType === null) {
            $this->setAuthType(new Plain($this));
        }

        return $this->authType;
    }

    public function setAuthType(Authenticable $authType): Options
    {
        $this->authType = $authType;

        return $this;
    }
}
