<?php

declare(strict_types=1);

namespace Pdahal\Xmpp;

use Pdahal\Xmpp\Exceptions\StreamError;
use Pdahal\Xmpp\Xml\Stanzas\Auth;
use Pdahal\Xmpp\Xml\Stanzas\Iq;
use Pdahal\Xmpp\Xml\Stanzas\Message;
use Pdahal\Xmpp\Xml\Stanzas\Presence;
use Pdahal\Xmpp\Xml\Xml;

class XmppClient
{
    use Xml;

    protected Socket $socket;

    public Auth $auth;

    public Iq $iq;

    public Presence $presence;

    public Message $message;

    public function __construct(protected Options $options)
    {
        $this->initDependencies();
        $this->initSession();
    }

    protected function initDependencies(): void
    {
        $this->socket = $this->initSocket();
        $this->initStanzas($this->socket);
    }

    public function connect(): void
    {
        $this->openStream();
        $this->auth->authenticate();
        $this->iq->setResource($this->options->getResource());
        $this->sendInitialPresenceStanza();
    }

    public function send(string $xml): void
    {
        $this->socket->send($xml);
    }

    public function getResponse(): string
    {
        $this->socket->receive();
        $response = $this->socket->getResponseBuffer()->read();

        return $this->checkForErrors($response);
    }

    public function prettyPrint(string $response): void
    {
        if ($response) {
            $separator = "\n-------------\n";
            echo "{$separator} {$response} {$separator}";
        }
    }

    public function disconnect(): void
    {
        $this->socket->send(self::closeXmlStream());
        $this->socket->disconnect();
    }

    protected function openStream(): void
    {
        $openStreamXml = self::openXmlStream($this->options->getDomain());
        $this->socket->send($openStreamXml);
    }

    protected function sendInitialPresenceStanza(): void
    {
        $this->socket->send('<presence/>');
    }

    protected function initStanzas(Socket $socket): void
    {
        $this->auth = new Auth($socket);
        $this->iq = new Iq($socket);
        $this->presence = new Presence($socket);
        $this->message = new Message($socket);
    }

    protected function initSession(): void
    {
        if (!headers_sent($filename, $linenum)) {
            if (session_status() === PHP_SESSION_NONE) {
                session_start();
            }
        }
    }

    protected function initSocket(): Socket
    {
        return new Socket($this->options);
    }

    /**
     * @throws StreamError
     */
    protected function checkForErrors(string $response): string
    {
        try {
            self::checkForUnrecoverableErrors($response);
        } catch (StreamError $e) {
            $this->options->getLogger()->logResponse(__METHOD__.'::'.__LINE__." {$response}");
            $this->options->getLogger()->error(__METHOD__.'::'.__LINE__.' '.$e->getMessage());
            $this->reconnect();
            $response = '';
        }

        return $response;
    }

    protected function reconnect(): void
    {
        $this->disconnect();
        $this->initDependencies();
        $this->connect();
    }
}
